@@ -3,22 +3,18 @@
  * This software is licensed under the "Simplified BSD License".
  * See LICENSE for details */
 
-#include "../src/ipc-protocol.h"
-#include "ipc-client.h"
-#include "client-utils.h"
-
-// standard
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
-#include <stdbool.h>
-#include <unistd.h>
-
-// gui
-#include <X11/Xlib.h>
-#include <X11/Xproto.h>
-#include <X11/Xutil.h>
+#include <X11/X.h>
 #include <X11/Xatom.h>
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
+#include <stdbool.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "../src/ipc-protocol.h"
+#include "client-utils.h"
+#include "ipc-client.h"
 
 struct HCConnection {
     Display*    display;
@@ -58,6 +54,9 @@ HCConnection* hc_connect_to_display(Display* display) {
 }
 
 void hc_disconnect(HCConnection* con) {
+    if (!con) {
+        return;
+    }
     if (con->client_window) {
         XDestroyWindow(con->display, con->client_window);
     }
@@ -91,7 +90,7 @@ bool hc_create_client_window(HCConnection* con) {
 }
 
 bool hc_send_command(HCConnection* con, int argc, char* argv[],
-                     GString** ret_out, int* ret_status) {
+                     char** ret_out, int* ret_status) {
     if (!hc_create_client_window(con)) {
         return false;
     }
@@ -104,7 +103,7 @@ bool hc_send_command(HCConnection* con, int argc, char* argv[],
     // get output
     int command_status = 0;
     XEvent event;
-    GString* output = NULL;
+    char* output = NULL;
     bool output_received = false, status_received = false;
     while (!output_received || !status_received) {
         XNextEvent(con->display, &event);
@@ -119,8 +118,8 @@ bool hc_send_command(HCConnection* con, int argc, char* argv[],
         }
         if (!output_received
             && pe->atom == con->atom_output) {
-            output = window_property_to_g_string(con->display, con->client_window,
-                                                 con->atom_output);
+            output = read_window_property(con->display, con->client_window,
+                                          con->atom_output);
             if (!output) {
                 fprintf(stderr, "could not get WindowProperty \"%s\"\n",
                                 HERBST_IPC_OUTPUT_ATOM);
@@ -152,13 +151,12 @@ bool hc_send_command(HCConnection* con, int argc, char* argv[],
 }
 
 bool hc_send_command_once(int argc, char* argv[],
-                          GString** ret_out, int* ret_status) {
+                          char** ret_out, int* ret_status) {
     HCConnection* con = hc_connect();
     if (con == NULL) {
         return false;
     }
     if (!hc_check_running(con)) {
-        fprintf(stderr, "Error: herbstluftwm is not running");
         return false;
     }
     bool status = hc_send_command(con, argc, argv, ret_out, ret_status);
@@ -216,7 +214,6 @@ bool hc_hook_window_connect(HCConnection* con) {
     }
     con->hook_window = get_hook_window(con->display);
     if (!con->hook_window) {
-        fprintf(stderr, "Error: herbstluftwm is not running");
         return false;
     }
     long mask = StructureNotifyMask|PropertyChangeMask;
