@@ -1,16 +1,10 @@
-/** Copyright 2011-2013 Thorsten Wißmann. All rights reserved.
- *
- * This software is licensed under the "Simplified BSD License".
- * See LICENSE for details */
-
 #ifndef __HERBSTLUFT_EWMH_H_
 #define __HERBSTLUFT_EWMH_H_
 
+#include <X11/X.h>
 #include <X11/Xlib.h>
-#include <X11/Xproto.h>
-#include <X11/Xatom.h>
-#include <stdbool.h>
 #include <array>
+#include <vector>
 
 #define ENUM_WITH_ALIAS(Identifier, Alias) \
     Identifier, Alias = Identifier
@@ -75,44 +69,6 @@ enum {
 #define _NET_WM_MOVERESIZE_MOVE_KEYBOARD    10   /* move via keyboard */
 #define _NET_WM_MOVERESIZE_CANCEL           11   /* cancel operation */
 
-struct HSTag;
-struct HSClient;
-
-extern Atom g_netatom[NetCOUNT];
-
-extern const std::array<const char*,NetCOUNT> g_netatom_names;
-
-void ewmh_init();
-void ewmh_destroy();
-void ewmh_update_all();
-
-void ewmh_add_client(Window win);
-void ewmh_remove_client(Window win);
-void ewmh_set_wmname(char* name);
-void ewmh_update_wmname();
-
-void ewmh_update_client_list();
-void ewmh_get_original_client_list(Window** buf, unsigned long *count);
-void ewmh_update_client_list_stacking();
-void ewmh_update_desktops();
-void ewmh_update_desktop_names();
-void ewmh_update_active_window(Window win);
-void ewmh_update_current_desktop();
-void ewmh_update_window_state(struct HSClient* client);
-void ewmh_update_frame_extents(Window win, int left, int right, int top, int bottom);
-bool ewmh_is_window_state_set(Window win, Atom hint);
-bool ewmh_is_fullscreen_set(Window win);
-bool ewmh_is_desktop_window(Window win);
-int ewmh_get_window_type(Window win); // returns an element of the NetWm-Enum
-void ewmh_clear_client_properties(struct HSClient* client);
-
-// set the desktop property of a window
-void ewmh_window_update_tag(Window win, struct HSTag* tag);
-
-void ewmh_handle_client_message(XEvent* event);
-
-void ewmh_set_window_opacity(Window win, double opacity);
-
 typedef enum {
     // see icccm:
     // http://www.x.org/releases/X11R7.7/doc/xorg-docs/icccm/icccm.html#WM_STATE_Property
@@ -121,7 +77,86 @@ typedef enum {
     WmStateIconicState    = 3,
 } WmState;
 
-void window_update_wm_state(Window win, WmState state);
+class HSTag;
+class Client;
+class Root;
+class TagManager;
+class XConnection;
+
+extern Atom g_netatom[NetCOUNT];
+
+extern const std::array<const char*,NetCOUNT> g_netatom_names;
+
+class Ewmh {
+public:
+    Ewmh(XConnection& xconnection);
+    ~Ewmh();
+
+    //! initial EWMH state
+    class InitialState {
+    public:
+        size_t numberOfDesktops = 0;
+        std::vector<std::string> desktopNames;
+        //! client list before hlwm start
+        std::vector<Window> original_client_list_;
+        void print(FILE* file);
+    };
+
+    enum class WM { Name, Protocols, Delete, State, TakeFocus, Last };
+
+    void injectDependencies(Root* root);
+    void updateAll();
+
+    void addClient(Window win);
+    void removeClient(Window win);
+    void updateWmName();
+
+    void updateClientList();
+    const InitialState &initialState();
+    long windowGetInitialDesktop(Window win);
+    void updateClientListStacking();
+    void updateDesktops();
+    void updateDesktopNames();
+    void updateActiveWindow(Window win);
+    void updateCurrentDesktop();
+    void updateWindowState(Client* client);
+    void updateFrameExtents(Window win, int left, int right, int top, int bottom);
+    bool isWindowStateSet(Window win, Atom hint);
+    bool isFullscreenSet(Window win);
+    void clearClientProperties(Window win);
+    std::string getWindowTitle(Window win);
+
+    int getWindowType(Window win);
+
+    bool isOwnWindow(Window win);
+    void clearInputFocus();
+
+    // set the desktop property of a window
+    void windowUpdateTag(Window win, HSTag* tag);
+
+    void handleClientMessage(XClientMessageEvent* me);
+
+    void setWindowOpacity(Window win, double opacity);
+
+    void windowUpdateWmState(Window win, WmState state);
+
+    static Ewmh& get(); // temporary singleton getter
+
+    bool sendEvent(Window window, WM proto, bool checkProtocols);
+    void windowClose(Window window);
+
+    XConnection& X() { return X_; }
+
+private:
+    bool focusStealingAllowed(long source);
+    Root* root_ = nullptr;
+    TagManager* tags_ = nullptr;
+    XConnection& X_;
+    InitialState initialState_;
+    void readInitialEwmhState();
+    Atom wmatom(WM proto);
+    Atom wmatom_[(int)WM::Last];
+};
 
 #endif
 
